#!/bin/bash

function is_ostree_system() {
    local immutable_system=$(installer_get "DI_IMMUTABLE_SYSTEM")
    if [[ $immutable_system == "true" ]]; then
        return 0
    else
        return 1
    fi
}

function ostree_get_deploy() {
    local commitID ostree ostree_refspec sysroot
    sysroot=$1
    ostree_refspec=$2
    commitID=$(ostree --repo=$sysroot/ostree/repo log "$ostree_refspec" | grep commit | head -n 1 | awk '{print $2}')
    ostree="/ostree/deploy/deepin/deploy/${commitID}.0"
    echo "$ostree"
}

function ostree_sys_init() {
    #rsync -azAX /ostree /target/
    unsquashfs_file "$DI_LIVE_DIR_ENV/live/filesystem.squashfs" /target/ ostree
    mkdir -p /target/boot || true

    local sysroot=/target
    local osname=deepin
    local ostree_refspec=$DI_REPO_BASE_NAME_ENV

    sysCommit=$(ostree rev-parse --repo=$sysroot/ostree/repo $ostree_refspec)
    info "sysCommit=$sysCommit"

    mkdir -p $sysroot/ostree/deploy/$osname/deploy || true
    ostree checkout --repo=$sysroot/ostree/repo $ostree_refspec "$sysroot/ostree/deploy/$osname/deploy/$sysCommit.0" || error "ostree_sys_init failed"
    info "ostree_sys_init ok"
}

function ostree_ext_mount() {
    local lowerdir=/target$1
    local ostree=$2
    local bootdir=$3
    local datadir=/target/persistent

    local mount_root_tool
    if which deepin-mount-root.sh > /dev/null 2>&1; then
        mount_root_tool=deepin-mount-root.sh
    elif which deepin-immutable-mount-root > /dev/null 2>&1; then
        mount_root_tool=deepin-immutable-mount-root
    else
        mount_root_tool=
        info "mount_root_tool not found"
    fi

    # 如果存在mount_root_tool，优先使用mount_root_tool挂载
    if [ -n "$mount_root_tool" ]; then
        local ostree_arg=$2
        mkdir -p $datadir/ostree/deploy/deepin/var
        mkdir -p $lowerdir/var || true
        $mount_root_tool /target $ostree || error "mount_root_tool /target $ostree failed"
        mount -o remount rw /target/usr || error "remount /target/usr failed"
        mount -o remount rw /target/etc || error "remount /target/etc failed"
        mount -o remount rw /target/opt || error "remount /target/opt failed"
        mount -o remount rw /target/persistent/ostree || error "remount /target/persistent/ostree failed"
        # 没有boot分区
        if [ -n "$bootdir" ]; then
            mkdir -p "$bootdir" || error "mkdir $bootdir failed"
        fi
    fi
}

function mount_partitions() {
    local mount_points
    mount_points=$(installer_get "DI_MOUNTS_POINTS")
    #mount_points="/dev/nvme0n1p3=/;/dev/nvme0n1p2=/boot;/dev/nvme0n1p1=/boot/efi"
    mount_points=(${mount_points//;/ })
    declare -A mount_map
    for p in "${mount_points[@]}"
    do
        kv=(${p//=/ })
        if [[ "${kv[1]}" != "swap" ]];then
            mount_map["${kv[1]}"]="${kv[0]}"
        fi
    done

    keys=("${!mount_map[@]}")
    sorted_keys=($(printf "%s\n" "${keys[@]}" | sort))
    for key in "${sorted_keys[@]}"; do
        local part="${mount_map[$key]}"
        if ! findmnt $part; then
            mkdir -p "/target/$key" || error "Failed to create mount point /target/$key"
            mount "${mount_map[$key]}" "/target/$key" || error "Failed to mount ${mount_map[$key]} /target/$key"
            echo "mount ${mount_map[$key]} /target$key"
        fi
    done
}
